# HealthRegen release 5 by David Bowland
# ./addons/eventscripts/healthregen/healthregen.py

# >>> To configure this addon please see healthregen.cfg <<<

"""
Provides steady health regeneration for all players up to an optional, maximum amount. Requires only ES 2.0+
"""


import es
import gamethread
import os.path

import psyco
psyco.full()


info = es.AddonInfo()
info.name     = 'HealthRegen'
info.version  = '5'
info.url      = 'http://addons.eventscripts.com/addons/view/healthregen'
info.basename = 'healthregen'
info.author   = 'SuperDave'


class RegenManager:
   userid    = 0
   has_delay = False
   remaining = 0

   def __init__(self, userid):
      """
      Stores userid for the instance
      Sets the maximum amount of health the player can regenerate
      """
      self.userid = userid

      round_max      = int(es.ServerVar('healthregen_roundmax'))
      self.remaining = round_max if round_max else -1

   def playerHurt(self):
      """Creates a delay to regenerate health for the player"""
      self.removeDelay()

      if self.remaining:
         self.has_delay = True

         delay = float(es.ServerVar('healthregen_start_delay'))
         gamethread.delayedname(delay if delay else float(es.ServerVar('healthregen_iteration_delay')), 'healthregen_%s' % self.userid, self._executeRegen)

   def removeDelay(self):
      """Removes the delay corresponding to the player"""
      if self.has_delay:
         gamethread.cancelDelayed('healthregen_%s' % self.userid)
         self.has_delay = False

   def _executeRegen(self):
      """
      Checks to make sure the player still needs health
      Adds the required health to the player and executes another iteration if the player needs health after addition
      """
      health_now = es.getplayerprop(self.userid, 'CBasePlayer.m_iHealth')
      health_max = int(es.ServerVar('healthregen_threshold'))

      if health_max > health_now:
         health_addition = int(es.ServerVar('healthregen_iteration_amount'))
         health_addition = health_addition if self.remaining == -1 else min(health_addition, self.remaining)

         health_now += health_addition
         if self.remaining <> -1:
            self.remaining -= health_addition

         if health_max > health_now:
            health_min = int(es.ServerVar('healthregen_start_health'))
            if self.remaining <> -1:
               if health_min - health_now > self.remaining:
                  health_min     = health_now + self.remaining
                  self.remaining = 0

               elif health_min - health_now > 0:
                  self.remaining -= health_min - health_now

            es.setplayerprop(self.userid, 'CBasePlayer.m_iHealth', health_min if health_min > health_now else health_now)

            if self.remaining:
               gamethread.delayedname(es.ServerVar('healthregen_iteration_delay'), 'healthregen_%s' % self.userid, self._executeRegen)

            else:
               self.has_delay = False

         else:
            es.setplayerprop(self.userid, 'CBasePlayer.m_iHealth', health_max)
            self.has_delay = False

      else:
         self.has_delay = False


players = {}


def getPlayer(userid):
   """Returns a RegenManager instance based on userid"""
   global players

   userid = int(userid)
   if not players.has_key(userid):
      players[userid] = RegenManager(userid)

   return players[userid]


def removePlayer(userid):
   """Removes a RegenManager instance based on userid"""
   global players

   userid = int(userid)
   if players.has_key(userid):
      players[userid].removeDelay()
      del players[userid]


def clearPlayers():
   """
   Removes all player delays
   Clears RegenManager instances
   """
   for userid in players:
      players[userid].removeDelay()

   players.clear()


###


dict_options = {'healthregen_threshold':[75, 'Maximum health that can be achieved with regeneration'], 'healthregen_roundmax':[0, 'Maximum amount of health that can be regenerated in a round--set to 0 for no limit'], 'healthregen_start_delay':[5, 'Number of seconds a player must remain unhurt to start regeneration--set to 0 for continuous regeneration'], 'healthregen_start_health':[40, 'Minimum amount of health players will have after the first regeneration iteration--set to 0 for no minimum'], 'healthregen_iteration_delay':[1, 'Seconds between regeneration iterations'], 'healthregen_iteration_amount':[2, 'Amount of health to regenerate each iteration']}

def load():
   """
   Checks to make sure critical server variables are created by the config
   """
   for str_option in dict_options:
      es.ServerVar(str_option, dict_options[str_option][0], dict_options[str_option][1])
   if os.path.isfile(es.getAddonPath('healthregen') + '/healthregen.cfg'):
      es.server.cmd('es_xmexec ../addons/eventscripts/healthregen/healthregen.cfg')
   else:
      es.dbgmsg(0, 'HealthRegen: Unable to load healthregen.cfg! Please ensure it is in the ./healthregen/ directory.')


def es_map_start(event_var):
   clearPlayers()


def round_start(event_var):
   clearPlayers()


def player_hurt(event_var):
   if es.ServerVar('healthregen_threshold') > int(event_var['es_userhealth']):
      getPlayer(event_var['userid']).playerHurt()


def player_death(event_var):
   removePlayer(event_var['userid'])


def player_disconnect(event_var):
   removePlayer(event_var['userid'])


def unload():
   clearPlayers()

   for str_option in dict_options:
      es.set(str_option, 0)